@@ -3,7 +3,7 @@
  * The University of Illinois/NCSA
  * Open Source License (NCSA)
  *
- * Copyright (c) 2017-2023, Advanced Micro Devices, Inc.
+ * Copyright (c) 2017-2025, Advanced Micro Devices, Inc.
  * All rights reserved.
  *
  * Developed by:
@@ -513,6 +513,9 @@ static const std::map<const char *, dev_depends_t> kDevFuncDependsMap = {
   {"rsmi_topo_numa_affinity_get",        {{kDevNumaNodeFName}, {}}},
   {"rsmi_dev_gpu_metrics_info_get",      {{kDevGpuMetricsFName}, {}}},
   {"rsmi_dev_gpu_reset",                 {{kDevGpuResetFName}, {}}},
+  {"rsmi_dev_energy_count_get",          {{kDevGpuMetricsFName}, {}}},
+  {"rsmi_dev_current_socket_power_get",  {{kDevGpuMetricsFName}, {}}},
+
   {"rsmi_dev_compute_partition_get",     {{kDevComputePartitionFName}, {}}},
   {"rsmi_dev_compute_partition_set",     {{kDevComputePartitionFName}, {}}},
   {"rsmi_dev_memory_partition_get",      {{kDevMemoryPartitionFName}, {}}},
@@ -661,6 +664,17 @@ int Device::openDebugFileStream(DevInfoTypes type, T *fs, const char *str) {
   return 0;
 }
 
+// The fallback sysfs to handle backward compatibilities
+static const std::map<DevInfoTypes, std::string> kDevFallbackFile = {
+  {kDevErrCntGFX, "ras/aca_gfx"},
+  {kDevErrCntSDMA, "ras/aca_sdma"},
+  {kDevErrCntUMC, "ras/aca_umc"},
+  {kDevErrCntMMHUB, "ras/aca_mmhub"},
+  {kDevErrCntPCIEBIF, "ras/aca_pcie_bif"},
+  {kDevErrCntHDP, "ras/aca_hdp"},
+  {kDevErrCntXGMIWAFL, "ras/aca_xgmi_wafl"},
+};
+
 template <typename T>
 int Device::openSysfsFileStream(DevInfoTypes type, T *fs, const char *str) {
   auto sysfs_path = path_;
@@ -680,6 +694,28 @@ int Device::openSysfsFileStream(DevInfoTypes type, T *fs, const char *str) {
   bool reg_file;
 
   int ret = isRegularFile(sysfs_path, &reg_file);
+
+  if (ret != 0 || !reg_file) {
+    // Handle specific types if the file does not exist
+    if (kDevFallbackFile.find(type) != kDevFallbackFile.end()) {
+
+      sysfs_path = path_ + "/device/" + kDevFallbackFile.at(type);
+      DBG_FILE_ERROR(sysfs_path, str);
+
+      // Recheck the adjusted path
+      ret = isRegularFile(sysfs_path, &reg_file);
+      if (ret != 0 || !reg_file) {
+        ss << __PRETTY_FUNCTION__
+           << " | Adjusted file path also does not exist - SYSFS file ("
+           << sysfs_path
+           << ") for DevInfoInfoType (" << get_type_string(type)
+           << "), returning " << std::to_string(ret);
+        LOG_ERROR(ss);
+        return ret;
+      }
+    }
+  }
+
   if (ret != 0) {
     ss << __PRETTY_FUNCTION__ << " | Issue: File did not exist - SYSFS file ("
        << sysfs_path
@@ -688,6 +724,7 @@ int Device::openSysfsFileStream(DevInfoTypes type, T *fs, const char *str) {
     LOG_ERROR(ss);
     return ret;
   }
+
   if (!reg_file) {
     ss << __PRETTY_FUNCTION__
        << " | Issue: File is not a regular file - SYSFS file ("
@@ -703,7 +740,7 @@ int Device::openSysfsFileStream(DevInfoTypes type, T *fs, const char *str) {
   if (!fs->is_open()) {
     ss << __PRETTY_FUNCTION__
        << " | Issue: Could not open - SYSFS file (" << sysfs_path << ") for "
-       << "DevInfoInfoType (" << get_type_string(type) << "), "
+       << "DevInfoTypes (" << get_type_string(type) << "), "
        << ", returning " << std::to_string(errno) << " ("
        << std::strerror(errno) << ")";
     LOG_ERROR(ss);
@@ -712,7 +749,7 @@ int Device::openSysfsFileStream(DevInfoTypes type, T *fs, const char *str) {
 
   ss << __PRETTY_FUNCTION__ << " | Successfully opened SYSFS file ("
      << sysfs_path
-     << ") for DevInfoInfoType (" << get_type_string(type)
+     << ") for DevInfoTypes (" << get_type_string(type)
      << ")";
   LOG_INFO(ss);
   return 0;
@@ -729,8 +766,8 @@ int Device::readDebugInfoStr(DevInfoTypes type, std::string *retStr) {
   ret = openDebugFileStream(type, &fs);
   if (ret != 0) {
     ss << "Could not read debugInfoStr for DevInfoType ("
-     << get_type_string(type) << "), returning "
-     << std::to_string(ret);
+       << get_type_string(type) << "), returning "
+       << std::to_string(ret);
     LOG_ERROR(ss);
     return ret;
   }
@@ -926,7 +963,7 @@ int Device::readDevInfoLine(DevInfoTypes type, std::string *line) {
      << get_type_string(type) << "), returning *line = "
      << *line;
   LOG_INFO(ss);
-
+  fs.close();
   return 0;
 }
 
@@ -1008,6 +1045,7 @@ int Device::readDevInfoMultiLineStr(DevInfoTypes type,
   while (std::getline(fs, line)) {
     retVec->push_back(line);
   }
+  fs.close();
 
   if (retVec->empty()) {
     ss << "Read devInfoMultiLineStr for DevInfoType ("
@@ -1388,7 +1426,6 @@ rsmi_status_t Device::restartAMDGpuDriver(void) {
   bool success = false;
   std::string out;
   bool wasGdmServiceActive = false;
-  bool restartInProgress = true;
   bool isRestartInProgress = true;
   bool isAMDGPUModuleLive = false;
   bool restartGDM = false;
@@ -1474,7 +1511,6 @@ rsmi_status_t Device::isRestartInProgress(bool *isRestartInProgress,
                                           bool *isAMDGPUModuleLive) {
   REQUIRE_ROOT_ACCESS
   std::ostringstream ss;
-  bool restartSuccessful = true;
   bool success = false;
   std::string out;
   bool deviceRestartInProgress = true;    // Assume in progress, we intend to disprove
@@ -1629,7 +1665,59 @@ std::string Device::readBootPartitionState<rsmi_memory_partition_type_t>(
   return boot_state;
 }
 
+rsmi_status_t Device::get_smi_device_identifiers(uint32_t device_id,
+        rsmi_device_identifiers_t *device_identifiers) {
+  bool found_device = false;
+  std::ostringstream ss;
+  rsmi_status_t ret = RSMI_STATUS_NOT_SUPPORTED;
+  if (device_identifiers == nullptr) {
+    return RSMI_STATUS_INVALID_ARGS;
+  }
+
+  amd::smi::RocmSMI& smi = amd::smi::RocmSMI::getInstance();
+  auto devices = smi.devices();
+  ss << __PRETTY_FUNCTION__ << " | device_id = " << device_id
+     << "; devices.size() = " << devices.size();
+  // std::cout << ss.str() << "\n";
+  LOG_DEBUG(ss);
+
+  for (uint32_t i = 0; i < devices.size(); i++) {
+    if (i != device_id) {
+      continue;
+    }
+
+    device_identifiers->card_index = devices[i]->index();
+    device_identifiers->drm_render_minor = devices[i]->drm_render_minor();
+    device_identifiers->bdfid = devices[i]->bdfid();
+    device_identifiers->kfd_gpu_id = devices[i]->kfd_gpu_id();
+    uint32_t temp_partition_id = 0;
+    rsmi_status_t ret = rsmi_dev_partition_id_get(
+        i, &temp_partition_id);
+    if (ret != RSMI_STATUS_SUCCESS) {
+      temp_partition_id = 0;
+    }
+    device_identifiers->partition_id = temp_partition_id;
+    device_identifiers->smi_device_id = i;
+    found_device = true;
+    ss << __PRETTY_FUNCTION__ << " | Found device: "
+       << "card_index = " << device_identifiers->card_index
+       << "; drm_render_minor = " << device_identifiers->drm_render_minor
+       << "; bdfid = " << std::hex << "0x" << device_identifiers->bdfid
+       << "; kfd_gpu_id = " << std::dec << device_identifiers->kfd_gpu_id
+       << "; partition_id = " << device_identifiers->partition_id
+       << "; smi_device_id = " << device_identifiers->smi_device_id;
+    // std::cout << ss.str() << "\n";
+    LOG_DEBUG(ss);
+    break;
+  }
+  if (found_device) {
+    ret = RSMI_STATUS_SUCCESS;
+  }
+  return ret;
+}
+
 
 #undef RET_IF_NONZERO
 }  // namespace smi
 }  // namespace amd
+
